@@ -100,21 +100,39 @@ public:
 
 class GVScrollPixmapView::ZoomToolController : public GVScrollPixmapView::ToolController {
 	QCursor mZoomCursor;
+private:
+	void zoomTo(const QPoint& pos, bool in) {
+		KAction* zoomAction=in?mView->zoomIn():mView->zoomOut();
+		if (!zoomAction->isEnabled()) return;
+		
+		if (mView->autoZoom()->isChecked()) {
+			mView->autoZoom()->setChecked(false);
+			mView->updateScrollBarMode();
+		}
+		QPoint centerPos=QPoint(mView->visibleWidth(), mView->visibleHeight())/2;
+
+		// Compute image position
+		QPoint imgPos=mView->viewportToContents(pos) - QPoint(mView->mXOffset, mView->mYOffset);
+		imgPos/=mView->zoom();
+		
+		double newZoom=mView->computeZoom(in);
+
+		imgPos*=newZoom;
+		imgPos=imgPos-pos+centerPos;
+		mView->setZoom(newZoom, imgPos.x(), imgPos.y());
+	}
+
 public:
 	ZoomToolController(GVScrollPixmapView* view) : ToolController(view) {
 		mZoomCursor=loadCursor("zoom");
 	}
 
-	void leftButtonReleaseEvent(QMouseEvent*) {
-		mView->zoomIn()->activate();
+	void leftButtonReleaseEvent(QMouseEvent* event) {
+		zoomTo(event->pos(), true);
 	}
 	
 	void wheelEvent(QWheelEvent* event) {
-		if (event->delta()<0) {
-			if (mView->zoomIn()->isEnabled()) mView->zoomIn()->activate();
-		} else {
-			if (mView->zoomOut()->isEnabled()) mView->zoomOut()->activate();
-		}
+		zoomTo(event->pos(), event->delta()<0);
 		event->accept();
 	}
 
@@ -295,11 +313,6 @@ void GVScrollPixmapView::slotModified() {
 	}
 }
 
-void GVScrollPixmapView::restartAutoHideTimer() {
-	mAutoHideTimer->start(AUTO_HIDE_TIMEOUT,true);
-}
-
-
 //------------------------------------------------------------------------
 //
 // Properties
@@ -380,10 +393,6 @@ void GVScrollPixmapView::setFullScreen(bool fullScreen) {
 	}
 }
 
-void GVScrollPixmapView::setButtonStateTool(ButtonState state, Tool tool) {
-	mButtonStateToolMap[state]=tool;
-}
-	
 
 //------------------------------------------------------------------------
 //
@@ -605,32 +614,20 @@ void GVScrollPixmapView::wheelEvent(QWheelEvent* event) {
 //
 //------------------------------------------------------------------------
 void GVScrollPixmapView::slotZoomIn() {
-	double newZoom;
-	if (mZoom>=1.0) {
-		newZoom=floor(mZoom)+1.0;
-	} else {
-		newZoom=1/( ceil(1/mZoom)-1.0 );
-	}
 	if (mAutoZoom->isChecked()) {
 		mAutoZoom->setChecked(false);
 		updateScrollBarMode();
 	}
-	setZoom(newZoom);
+	setZoom(computeZoom(true));
 }
 
 
 void GVScrollPixmapView::slotZoomOut() {
-	double newZoom;
-	if (mZoom>1.0) {
-		newZoom=ceil(mZoom)-1.0;
-	} else {
-		newZoom=1/( floor(1/mZoom)+1.0 );
-	}
 	if (mAutoZoom->isChecked()) {
 		mAutoZoom->setChecked(false);
 		updateScrollBarMode();
 	}
-	setZoom(newZoom);
+	setZoom(computeZoom(false));
 }
 
 
@@ -661,6 +658,11 @@ void GVScrollPixmapView::setAutoZoom(bool value) {
 // Private
 //
 //------------------------------------------------------------------------
+void GVScrollPixmapView::restartAutoHideTimer() {
+	mAutoHideTimer->start(AUTO_HIDE_TIMEOUT,true);
+}
+
+
 void GVScrollPixmapView::openContextMenu(const QPoint& pos) {
 	QPopupMenu menu(this);
 
@@ -759,7 +761,7 @@ static void sizeScaleMin(QSize* size, int w, int h) {
 }
 #endif
 
-double GVScrollPixmapView::computeAutoZoom() {
+double GVScrollPixmapView::computeAutoZoom() const {
 
 	if (mGVPixmap->isNull()) {
 		return 1.0;
@@ -778,6 +780,23 @@ double GVScrollPixmapView::computeAutoZoom() {
 }
 
 
+double GVScrollPixmapView::computeZoom(bool in) const {
+	if (in) {
+		if (mZoom>=1.0) {
+			return floor(mZoom)+1.0;
+		} else {
+			return 1/( ceil(1/mZoom)-1.0 );
+		}
+	} else {	
+		if (mZoom>1.0) {
+			return ceil(mZoom)-1.0;
+		} else {
+			return 1/( floor(1/mZoom)+1.0 );
+		}
+	}
+}
+
+
 void GVScrollPixmapView::updateImageOffset() {
 	int viewWidth=width();
 	int viewHeight=height();
@@ -787,10 +806,10 @@ void GVScrollPixmapView::updateImageOffset() {
 	int zpixWidth=int(mGVPixmap->width() * mZoom);
 	int zpixHeight=int(mGVPixmap->height() * mZoom);
 
-	if (zpixWidth>viewWidth) {
+	if (zpixWidth>viewWidth && hScrollBarMode()!=AlwaysOff) {
 		viewHeight-=horizontalScrollBar()->height();
 	}
-	if (zpixHeight>viewHeight) {
+	if (zpixHeight>viewHeight && vScrollBarMode()!=AlwaysOff) {
 		viewWidth-=verticalScrollBar()->width();
 	}
 	
